@@ -23,6 +23,13 @@ from os.path import split
 
 from .compat import PY3
 
+# This is here (in the utils module) because it might be used by
+# various other modules.
+try:
+    from pathlib2 import Path   # noqa: F401
+except ImportError:
+    from pathlib import Path    # noqa: F401
+
 try:
     from urllib.parse import urlparse, parse_qs
 except ImportError:
@@ -51,7 +58,7 @@ except ImportError:
 
 TIME_UNITS = {
     "": "Seconds",
-    "m": "Miliseconds (ms)",
+    "m": "Milliseconds (ms)",
     "u": "Microseconds (us)",
     "n": "Nanoseconds (ns)"
 }
@@ -101,49 +108,53 @@ def get_machine_id():
 
 
 class Fallback(object):
-    def __init__(self, *exceptions):
+    def __init__(self, fallback, exceptions):
+        self.fallback = fallback
         self.functions = []
         self.exceptions = exceptions
 
     def __call__(self, *args, **kwargs):
         for func in self.functions:
             try:
-                return func(*args, **kwargs)
+                value = func(*args, **kwargs)
             except self.exceptions:
                 continue
+            else:
+                if value:
+                    return value
         else:
-            raise NotImplementedError("Must have an fallback implementation for %s" % self.functions[0])
+            return self.fallback(*args, **kwargs)
 
     def register(self, other):
         self.functions.append(other)
         return self
 
 
-get_project_name = Fallback(IndexError, CalledProcessError, OSError)
+@partial(Fallback, exceptions=(IndexError, CalledProcessError, OSError))
+def get_project_name():
+    return basename(os.getcwd())
 
 
 @get_project_name.register
 def get_project_name_git():
-    project_address = check_output(['git', 'config', '--local', 'remote.origin.url'])
-    if isinstance(project_address, bytes) and str != bytes:
-        project_address = project_address.decode()
-    project_name = [i for i in re.split(r'[/:\s\\]|\.git', project_address) if i][-1]
-    return project_name.strip()
+    is_git = check_output(['git', 'rev-parse', '--git-dir'], stderr=subprocess.STDOUT)
+    if is_git:
+        project_address = check_output(['git', 'config', '--local', 'remote.origin.url'])
+        if isinstance(project_address, bytes) and str != bytes:
+            project_address = project_address.decode()
+        project_name = [i for i in re.split(r'[/:\s\\]|\.git', project_address) if i][-1]
+        return project_name.strip()
 
 
 @get_project_name.register
 def get_project_name_hg():
-    project_address = check_output(['hg', 'path', 'default'])
+    with open(os.devnull, 'w') as devnull:
+        project_address = check_output(['hg', 'path', 'default'], stderr=devnull)
     project_address = project_address.decode()
     project_name = project_address.split("/")[-1]
     return project_name.strip()
 
 
-@get_project_name.register
-def get_project_name_default():
-    return basename(os.getcwd())
-
-
 def in_any_parent(name, path=None):
     prev = None
     if not path:
@@ -278,9 +289,9 @@ class DifferenceRegressionCheck(RegressionCheck):
 
 
 def parse_compare_fail(string,
-                       rex=re.compile('^(?P<field>min|max|mean|median|stddev|iqr):'
-                                      '((?P<percentage>[0-9]?[0-9])%|(?P<difference>[0-9]*\.?[0-9]+([eE][-+]?['
-                                      '0-9]+)?))$')):
+                       rex=re.compile(r'^(?P<field>min|max|mean|median|stddev|iqr):'
+                                      r'((?P<percentage>[0-9]?[0-9])%|(?P<difference>[0-9]*\.?[0-9]+([eE][-+]?['
+                                      r'0-9]+)?))$')):
     m = rex.match(string)
     if m:
         g = m.groupdict()
@@ -329,10 +340,18 @@ def name_formatter_long(bench):
         return bench["fullname"]
 
 
+def name_formatter_trial(bench):
+    if bench["source"]:
+        return "%.4s" % split(bench["source"])[-1]
+    else:
+        return '????'
+
+
 NAME_FORMATTERS = {
     "short": name_formatter_short,
     "normal": name_formatter_normal,
     "long": name_formatter_long,
+    "trial": name_formatter_trial,
 }
 
 
@@ -568,7 +587,7 @@ def report_noprogress(iterable, *args, **kwargs):
 
 
 def slugify(name):
-    for c in "\/:*?<>| ":
+    for c in r"\/:*?<>| ":
         name = name.replace(c, '_').replace('__', '_')
     return name
 
